<?php defined('SYSPATH') or die('No direct script access.');
/*
 * This file is part of open source system FreenetIS
 * and it is released under GPLv3 licence.
 * 
 * More info about licence can be found:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * 
 * More info about project can be found:
 * http://www.freenetis.org/
 * 
 */

/**
 * This upgrade is not equal to any of previos upgrades
 * 
 * @author Ondřej Fibich <ondrej.fibich@gmail.com>
 */
$upgrade_equal_to['1.1.0~alpha12'] = array();

/**
 * Speed classes (#359) and SMS table optimalization (#412).
 * 
 * @author Ondřej Fibich <ondrej.fibich@gmail.com>
 */
$upgrade_sql['1.1.0~alpha12'] = array
(
	/* SMS table optimalization (#419) */
	"ALTER TABLE `sms_messages` ADD INDEX (`sender`);",
	"ALTER TABLE `sms_messages` ADD INDEX (`receiver`);",
	
	/* Speed classes (#359) */
	"CREATE TABLE  `speed_classes` (
	  `id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	  `name` VARCHAR( 50 ) NOT NULL ,
	  `d_ceil` BIGINT( 11 ) NOT NULL COMMENT  'QoS download ceil in bytes',
	  `d_rate` BIGINT( 11 ) NOT NULL COMMENT  'QoS download rate in bytes',
	  `u_ceil` BIGINT( 11 ) NOT NULL COMMENT  'QoS upload ceil in bytes',
	  `u_rate` BIGINT( 11 ) NOT NULL COMMENT  'QoS upload rate in bytes',
	  `regular_member_default` TINYINT(1) NOT NULL DEFAULT 0 COMMENT 'Is this class default for regular members?',
	  `applicant_default` TINYINT(1) NOT NULL DEFAULT 0 COMMENT 'Is this class default for applicants?',
	  PRIMARY KEY ( `id` )
	 ) ENGINE = INNODB COMMENT =  'Defines speed classes for QoS';",
	// relation between members and speed classes
	"ALTER TABLE  `members` ADD  `speed_class_id` INT( 11 ) DEFAULT NULL AFTER  `qos_rate`",
	"ALTER TABLE `members` ADD FOREIGN KEY `speed_class_id_fk` (`speed_class_id`)
	 REFERENCES `speed_classes` (`id`) ON DELETE SET NULL",
);

/**
 * Transform old QoS data to speed classes and remove old columns
 * 
 * @return boolean
 */
function upgrade_1_1_0_alpha12_after()
{
	$db = Database::instance();
	$member_model = new Member_Model();
	$speed_classes = array();
	
	try
	{
		$member_model->transaction_start();

		// transform old data
		$members = $member_model->find_all();
		$speed_class = new Speed_class_Model();

		foreach ($members as $member)
		{
			if (valid::speed_size($member->qos_ceil) &&
				valid::speed_size($member->qos_rate))
			{
				$d_ceil = $u_ceil = $member->qos_ceil;
				$d_rate = $u_rate = $member->qos_rate;

				if (strpos($member->qos_ceil, '/'))
				{
					$u_ceil = substr($u_ceil, 0, strpos($u_ceil, '/'));
					$d_ceil = substr($d_ceil, strpos($d_ceil, '/') + 1);
				}

				if (strpos($member->qos_rate, '/'))
				{
					$u_rate = substr($u_rate, 0, strpos($u_rate, '/'));
					$d_rate = substr($d_rate, strpos($d_rate, '/') + 1);
				}
				
				$bu_ceil = network::str2bytes($u_ceil);
				$bd_ceil = network::str2bytes($d_ceil);
				$bu_rate = network::str2bytes($u_rate);
				$bd_rate = network::str2bytes($d_rate);
				$index = $bu_ceil . '_' - $bd_ceil . '_' . $bu_rate . '_' . $bd_rate;

				if (!isset($speed_classes[$index]))
				{
					$speed_class->clear();
					$speed_class->name = __('Speed class') . ' '
							. network::speed($bd_ceil) . '/'
							. network::speed($bu_ceil) . ' - '
							. network::speed($bd_rate) . '_'
							. network::speed($bu_rate);
					$speed_class->d_rate = $bd_rate;
					$speed_class->d_ceil = $bd_ceil;
					$speed_class->u_rate = $bu_rate;
					$speed_class->u_ceil = $bu_ceil;
					$speed_class->applicant_default = FALSE;
					$speed_class->regular_member_default = FALSE;
					$speed_class->save_throwable();
					
					$speed_classes[$index] = $speed_class->id;
				}
				
				$member->speed_class_id = $speed_classes[$index];
			}
			
			$member->save_throwable();
		}

		// delete old columns
		$db->query("ALTER TABLE `members` DROP `qos_ceil`, DROP `qos_rate`");

		$member_model->transaction_commit();
	}
	catch (Exception $e)
	{
		$member_model->transaction_rollback();
		throw $e;
	}
	
	return TRUE;
}
